# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2004,2006 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# anytype _csa(string what)
#
# Looks up several pieces of information regarding the current program
# and CSA layer in general, and returns it to the caller. This function
# provides an easy way to assess variables and settings, especially
# those that may not be comfortably obtained through simple ENVIRON[]
# lookups and other straightforward techniques.
# =====================================================================

function _csa(what,		s) {

  # Now a null pgm value indicates a generic map lookup.

  if (what == "progname") {
     s = ENVIRON["CSA_PGM"]
     sub(/.*\001/,_NULL,s)		# keep only leftmost value.
     return s
  }

  # Test whether CSA is running in interactive mode.

  else if (what == "interactive" && \
    ENVIRON["CSA_PGM"] ~ /^CSAI/) return _TRUE

  # Return _TRUE if an interactive Web action requiring confirmation was
  # confirmed by the client, or _FALSE otherwise. Note that if CSA is
  # running in non-interactive mode, then this test always returns _TRUE.

  else if (what == "confirmed" && \
    (_bool(ENVIRON["CSA_CONFIRMED"]) == _TRUE || \
    _csa("interactive") == _FALSE)) return _TRUE

  # Return _TRUE if an idempotent HTTP method was requested.
  else if (what == "idempotent" && \
    ENVIRON["REQUEST_METHOD"] ~ /^(GET|HEAD)$/) return _TRUE

  # Return _TRUE if CSA Processing Instructions (CPI) are allowed.
  else if (what == "cpi" && \
    _bool(ENVIRON["CSA_ENABLE_CPI"]) == _TRUE) return _TRUE

  # add more 'else if' clauses as needed.
  return _FALSE
}

