# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2001,2007 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# string _isodate(string local_date)
#
# Takes a date string, in the local format, and returns the corresponding
# ISO format "YYYY-mm-dd HH:MM:SS". No checks are done to make sure that
# the specified date is a valid one. The input date string is interpreted
# based on CSA_LANG. If the input date lacks the HH:MM:SS part, then
# the latter defaults to "00:00:00" in the returned value. The function
# can cope with day and/or month values specified with only one digit,
# i.e. 3/12/2000, 11/3/2000, 3/5/2000 and that. The same with HH:MM:SS.
# =====================================================================

function _isodate(date,			      iso_date,token,a,b,i,j) {

  iso_date = _strip(date)
  if (iso_date == _NULL) return _NULL

  if (split(date,a," ") < 2) a[2] = "00:00:00"

  else {
     i = (split(a[2],b,":"))

     a[2] = ""

     # front-pad with zero if necessary.
     for (j=1; j<=i; j++) {
	 if (length(b[j]"") < 2) b[j] = "0" b[j]
	 if (j == 1) a[2] = b[j]
	 else a[2] = a[2] ":" b[j]
     }
  }

  # Add seconds if only HH:MM are present.
  if (a[2] ~ /^[0-2][0-9]:[0-5][0-9]$/) a[2] = a[2] ":00"

  if (split(a[1], token, /[-\/ :.]/) != 3) return _NULL

  # Front-pad with zero if necessary.
  if (length(token[1]"") < 2) token[1] = "0" token[1]
  if (length(token[2]"") < 2) token[2] = "0" token[2]

  if (ENVIRON["CSA_LANG"] != "en_US") {
     iso_date = token[3] "-" token[2] "-" token[1]
  }
  else {					    # Default is "en_US"
     iso_date = token[3] "-" token[1] "-" token[2]
  }
  return iso_date " " a[2]
}
