# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2007,2008 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# string _parsecpi(string s)
#
# Parse CSA processing instructions in "s".
# =====================================================================

function _parsecpi(s) {

  # =========================================
  # conditional output: (::DEL:) ... (:DEL::)
  # =========================================

  # the "usual" trick of low ASCII characters used as asymmetrical
  # single-character markers, to make block-tags nestable.

  gsub(/\(::DEL:\)/,"\001",s)
  gsub(/\(:DEL::\)/,"\002",s)

  # The next iterative replacement is more costly in terms of CPU
  # cycles than a gsub() but it is needed to handle nesteable metatags,
  # such as (::DEL:), (:DEL::), etc. On the other hand, the complete
  # parsing of the input string is going to be much more costly as
  # new tags are introduced, both by CSA and by application programs,
  # and this loop becomes rather irrelevant compared with the former.

  while (sub(/\001[^\001\002]*\002/,"",s));

  # =========================================
  # National language being used: (:LANG:)
  # =========================================

  gsub(/\(:LANG:\)/,ENVIRON["CSA_LANG"],s)

  # Add support for further tags below.
  # ...

  # get rid of any orphan markers (this must come last).
  gsub(/[\001\002]/,"",s)

  return s
}

