# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2001,2007 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# void _rdfprint(string s, boolean parse, [string ofile])
#
# Special "print" function for XML/RDF templates. Note that for this
# function to work correctly, the input template must be correctly
# formatted, that is RDF begin and end tags must be on separate lines;
# the same applies to 'csa:orb' begion and end tags.
# =====================================================================

function _rdfprint(s,parse,ofile,nocpi,		    c,t,re1,re2,skip) {

  # Have we been told to begin printing after
  # a certain no. of subsequent calls ?

  if (_RDFPRINT > 1) {
     _RDFPRINT--
     return
  }

  # Set defaults.
  if (ofile == "") ofile = _STDOUT

  if (ENVIRON["CSA_RDF_ORB"] != "/dev/null") {
     re1 = "^[ \t]*<csa:orb "
     re2 = "^[ \t]*</csa:orb>$"
     skip = 1
  }
  else {
     re1 = "^[ \t]*<rdf:RDF "
     re2 = "^[ \t]*</rdf:RDF>$"
  }

  # Note: do XML-encoding first (if requested), and then CSA tag
  # parsing, not the other way around!

  if (parse) s = _xmlencode(s)
  if (nocpi) gsub(/\(:/,"\\&#40;:",s)

  if (tolower(ENVIRON["CSA_RPC_FMT"]) ~ /^(rdf|rss)$/) {
     if (_XMLTOP != _NULL) _RDFPRINT = 0
     if (s ~ /^<\?xml [^>]+\?>\r?$/) {
	# Use a simplified XML header if ORB.
	if (ENVIRON["CSA_RDF_ORB"] != "/dev/null")
	   s = sprintf("<?xml version=\"1.0\" encoding=\"%s\"?>",\
				_xmlencode(ENVIRON["CSA_CHARSET"]))
	printf("%s\n",s) > ofile
	_XMLTOP = _NULL			# the template provided its own.
	_RDFPRINT = 0
	return
     }
     else if (s ~ re1) {
	sub(/^[ \t]*/,"",s)		# XML clients may be picky.
	if (_XMLTOP != _NULL) {		# the template did not provide one.

	   # Use a simplified XML header if ORB.
	   if (ENVIRON["CSA_RDF_ORB"] != "/dev/null")
	      _XMLTOP = sprintf("<?xml version=\"1.0\" encoding=\"%s\"?>",\
					_xmlencode(ENVIRON["CSA_CHARSET"]))
	   print _XMLTOP > ofile
	   _XMLTOP = _NULL
	}
	if (skip) {
	   # Delay printing by 1, to skip the trailing newline.
	   _RDFPRINT = 2
	   return
	}
	_RDFPRINT = 1
     }
     else if (s ~ re2) {
	sub(/^[ \t]*/,"",s)		# XML clients may be picky.
	if (!skip) printf("%s\n",s) > ofile
	_RDFPRINT = 0
	return
     }
  }

  # MSIE 5/6 does not behave appropriately if the response begins 
  # with <?xml ...?> . On the other hand I cannot remove the line
  # completely, due to how _envtoxml() calls _rdfprint().

  if (ENVIRON["HTTP_USER_AGENT"] ~ / MSIE [56]/ && \
	s ~ /^<\?xml [^>]+\?>\r?$/) s = "<!-- MSIE 5/6 buggery -->"

  if (_RDFPRINT == 1) printf("%s", s) > ofile
}
