# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2004,2006 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================

# =====================================================================
# int _sendmail(string template [,string eval,boolean trim,\
#					int lvl, boolean norepl])
#
# Send an e-mail message from within a _userproc(_O_RESPONSE) context,
# based on a message template file.
# =====================================================================

function _sendmail(tpl,eval,trim,lvl,norepl,	      		      \
	   		ltago,ltagc,retago,retagc,nl,end,tlen,tag,enc,\
			a,e,i,j,t,T,input,record,start,tago,tagc) {

  i=j=0; norepl/=1

  # Check args and set defaults.
  if (tpl == _NULL) return(1)				# no default!
  if (tpl !~ /^\//)
     tpl = ENVIRON["CSA_ROOT"] "/forms/" ENVIRON["CSA_LANG"] "/" tpl
  tago = "$["						# fixed.
  tagc = "]"						# fixed.
  ltago = 2						# fixed.
  ltagc = 1						# fixed.

  if (!ENVIRON["CSA_TPLMAX"]) ENVIRON["CSA_TPLMAX"] = 10

  # prevent infinite recursions.
  if (lvl > ENVIRON["CSA_TPLMAX"]) return 1

  # Prepare the substitution regexp's.
  retago = _escreg(tago)
  retagc = _escreg(tagc)

  retago = retago "[-A-Za-z0-9_/.]+(:[a-zA-Z]*)?" retagc

  if (!lvl) {				# only if 1st recursion.

     for (j in ENVIRON) {

	 if (j == _NULL) continue

	 # Always skip rc(1) functions, confidential data
	 # and other CSA-specific stuff.

	 if (j ~ ENVIRON["CSA_TPL_DENY"]) {
	    ENVIRON[j] = "***removed***"
	    continue
	 }

	 # If CSA_AUTH_SSL is in effect, do not include
	 # HTTP_AUTHORIZATION in mail templates.

	 if (j == "HTTP_AUTHORIZATION" && ENVIRON["CSA_AUTH_SSL"]) {
	    ENVIRON[j] = ""
	    continue
	 }
     }

     if (_CMD != _NULL) close(_CMD)		# just in case.

     _CMD = ENVIRON["CSA_CMD_SENDMAIL_"]	# open MTA command.
  }

  # I always append CSA_TPL_EVAL, in case this function is called
  # directly, as opposed to being called by csaSendMail. If the
  # same files are listed multiple times in the resulting 'eval'
  # variable, it doesn't matter.

  if (eval != "") eval = eval ","
  eval = eval ENVIRON["CSA_TPL_EVAL"]

  if (eval != "") split(eval,a,",")

  # Process the template file.

  while (getline input < tpl > 0) {
     if (input ~ /^#/) continue	    		# skip comment lines.
     nl = 1
     do {
        match(input,retago)
        if (RSTART) {
	   end = index(input, tagc)

	   # account for an "orphan" tagc before tago,
	   # i.e. "...] something $[....] something else ..."

	   if (end && end < RSTART) {
	      printf("%s",substr(input, 1, end)) | _CMD
	      input = substr(input, end + 1)
	      continue
	   }

	   tlen = end - RSTART - ltago
	   start = RSTART
	   printf("%s",substr(input, 1, start - 1)) | _CMD
	   tag = substr(input, start + ltago, tlen)

	   # Extract encoding flags, if any.

	   if (tag ~ /:[a-zA-Z]+$/) {
	      enc = tag
	      sub(/.*:/,_NULL,enc)
	      sub(/:[a-zA-Z]+$/,_NULL,tag)
	   }

	   else enc = _NULL			# mandatory!

	   # Since AWK can open only one _CMD at a time, no system()
	   # are allowed here, so no encoding flags that imply calling
	   # an external program are supported in file tags. On the
	   # other hand it is trivial for the calling program to prepare
	   # pre-encoded binary attachments with 'swu-encdec -e -b' or
	   # something to that effect.

	   for (i in a) {			# Is "tag" to be eval'ed ?
	       if (a[i] == tag) {
		  t = T = tag
		  gsub(/\./,"__2e",t)		# Allow rc(1) encoded dots.
		  tag = ENVIRON[t]		# Get tag value from env.

		  # Get also new encoding flag from env, if available,
		  # otherwise stick with whatever encoding was defined
		  # in the original tag.

		  if (tag ~ /:[a-zA-Z]+$/) {
		     enc = tag
		     sub(/.*:/,_NULL,enc)
		     sub(/:[a-zA-Z]+$/,_NULL,tag)
		  }

		  # Evaluation of non-file tags is currently limited
		  # to the encoding flags. If this is not a file-tag,
		  # then restore the original file name and set the
		  # new value to the old one, but without the flags.

		  if (tag !~ /^-?\.?\//) {
		     ENVIRON[t] = tag
		     tag = T
		  }

		  break
	       }
	   }

	   if (tag ~ /^-?\.?\//) {		# Catch file patterns.
	      # Catch any '..' pattern.
	      if (tag ~ /\.\./) {
		 printf("%s%s%s",tago,tag,tagc) | _CMD	# Print tag literally.
	      }

	      # Remember: any binary stuff is required to have
	      # already been b64-encoded by the caller!

	      else {

		 # Switch to no-replace mode if file tag begins with "-"
		 if (sub(/^-/,_NULL,tag)) norepl = 1

		 gsub(/\/\//, "/", tag)		# Turn any "//" into "/".

		 # Only one single encoding flag, if any, is accepted
		 # in case of file tag. Add more flags here as needed,
		 # provided that they do not require an external program
		 # for encoding.

		 if (enc == "c") {
		    # embeddable character-data, include verbatim.
		    _sendmail(tag,eval,trim,lvl+1,norepl)
		 }
		 else {
		    # Currently the default encoding flag is "c",
		    # i.e. the same as above.
		    _sendmail(tag,eval,trim,lvl+1,norepl)
		 }
		 nl = norepl = 0
	      }
	   }
	   else {

	      if (norepl) value = tago tag tagc
	      else {
		 t = tag
		 # un-encode rc(1) dots, in case tag was eval'ed.
		 gsub("__2e",".",t)
		 if (t ~ ENVIRON["CSA_TPL_ALLOW"]) {
		    # Allow rc(1) encoded dots in tag name.
		    gsub(/\./,"__2e",tag)
		    value = ENVIRON[tag]
		 }
		 else value = "***removed***"
	      }

	      # Perform encoding, if requested. Note that most of
	      # these encoding types are currently mainly meant to
	      # properly encode URIs inside e-mail messages.

	      i = split(enc,e,_NULL)
	      for (j=1; j<=i; j++) {
		  if (e[j] == "i") value = _isoencode(value)
		  else if (e[j] == "j") value = _jsencode(value)
		  else if (e[j] == "u") value = _uriencode(value)
		  else if (e[j] == "x") value = _xmlencode(value)
		  else if (e[j] == "P") value = _uriencode(value,1)
	      }
	      enc = _NULL			# clear for next loop!

	      if (trim) value = _strip(value)
	      printf("%s",value) | _CMD
	   }

	   input = substr(input, end + ltagc)
	}
	else {
	   printf("%s",input) | _CMD
	   input = _NULL		# Not to loop forever.
	}
     }  while (length(input))

     if (nl) printf("\n") | _CMD
  }

  close(tpl)				# close input.

  if (!lvl) close(_CMD)			# send message if at top level.
}

