# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2004,2006 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# int _sys(string shell [, string ofile])
#
# Write the specified shell statements to $CSA_RPC_WWW by default,
# or to whatever file passed in the 2nd argument. The "shell" parameter
# must be suitable to be safely eval'ed by the shell, or to whatever
# interpreter 'ofile' is to be executed by. If this function is called
# while processing a response rather than a request, then the default
# output file is $CSA_TPLWB as opposed to $CSA_RPC_WWW.
#
# Special care should be taken when calling this function from within
# the response section of _userproc(), to write a 'csaExit.*' statement 
# to the underlying $CSA_TPLWB script. That is, the only 'csaExit.*'
# types that can be safely used in that context to "bail-out" from the
# current CGI program are 'csaExit.fault' and 'csaExit.location'; all
# all other 'csaExit.*' types my raise a number of issues when run by
# the underlying CSA_TPLWB script, like duplicated HTTP headers and
# endless loops in some cases.
#
# Note that _userproc() should normally call this function as an
# argument to return(), i.e. either as 'return(_sys(args))' or as
# 'return _sys(args)'.
# =====================================================================

function _sys(shell,ofile) {

  if (ofile == _NULL) {

     if (ENVIRON["CSA_TPLWB"] == _NULL) {
	if (ENVIRON["CSA_RPC_WWW"] != _NULL)
			ofile=ENVIRON["CSA_RPC_WWW"]	# web request
	else ofile = _STDOUT				# batch request
     }

     # response
     else {
	ofile=ENVIRON["CSA_TPLWB"]
	_RESULT = _ENOMSG			# tell _envtoxml().
     }
  }
  
  if (ofile !~ /^\/\.*[a-zA-Z0-9]/) return _EINVAL

  print shell > ofile
}

