# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2007 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# string _uridecode(string s, boolean mode)
#
# Perform URI-decoding of special characters, according to RFC 1378. If
# the 'mode' argument is set to '1', then the decoding is done twice for
# some of the involved characters. This is meant to correctly decode
# values that where encoded with _uriencode(string,_O_PATHINFO), if
# those values have been received through other than through an HTTP
# URL. This is the case, for istance, of values encoded with said
# option, that are sent back to us by the client through a different
# API, like an XML-RPC parameter list or something else. See also the
# notes about the same flag in _uriencode(). Note that where thing are
# received through PATH_INFO, that is over an URL as it is usually the
# case,_uridecode() must NOT be passed the _O_PATHINFO flag because a
# first decoding pass will already have been done by the web server.
# =====================================================================

function _uridecode(string,mode,		d,j,k,key,repl) {

   # do not waste time if there are no "%" characters in string.
   if ((j=split(string,d,"%")) < 2) {
      gsub(/\+/, " ", string)
      return string
   }

   # Disable selected character mappings which need to be done
   # in an properly ordered sequence after all other translations
   # have occurred.

   _CHARMAP["2B"] = _CHARMAP["2b"] = _NULL
   _CHARMAP["25"] = _NULL

   # identify escape codes and perform order-independent conversions.
   string = d[1]
   k = 1
   while(++k <= j) {
     key = substr(d[k],1,2)
     repl = _CHARMAP[key]

     # restore original code if lookup failed.
     if (repl == "") repl = "%" key

     string = string repl substr(d[k],3)
   }

   # perform order-dependent conversions last.
   gsub(/\+/, " ", string)
   gsub(/%2[Bb]/, "+", string)
   gsub(/%25/, "%", string)

   # Re-enable global mappings that had temporarily been disabled!

   _CHARMAP["2B"] = _CHARMAP["2b"] = "\053"
   _CHARMAP["25"] = "\045"

   # When a value sent to a client is expected to be received over a
   # URL, some characters need to be encoded twice, because they will
   # be decoded a first time by the web server itself. When a client
   # sends those values back to the CSA application through other than
   # a URL (for instance in the HTTP request body), the application
   # needs to tell this function to decode those characters twice,
   # because they have not been through the web server decoding step.

   if (mode == _O_PATHINFO) {
      gsub(/%26/, "\\&", string)
      gsub(/%3[Ff]/, "?", string)
      gsub(/%3[Dd]/, "=", string)
      gsub(/%2[Ff]/, "/", string)
   }

   return string
}

